@@ -50,6 +50,8 @@
 
 """CLI interface for cinder management."""
 
+from __future__ import annotations
+
 import collections
 import collections.abc as collections_abc
 import errno
@@ -60,6 +62,8 @@ import os
 import re
 import sys
 import time
+import typing
+from typing import Any, Callable, Optional, Tuple, Union
 
 from oslo_config import cfg
 from oslo_db import exception as db_exc
@@ -101,6 +105,7 @@ OVO_VERSION = ovo_base.OBJ_VERSIONS.get_current()
 
 
 # Decorators for actions
+@typing.no_type_check
 def args(*args, **kwargs):
     args = list(args)
     if not args[0].startswith('-') and '-' in args[0]:
@@ -118,7 +123,7 @@ class HostCommands(object):
 
     @args('zone', nargs='?', default=None,
           help='Availability Zone (default: %(default)s)')
-    def list(self, zone=None):
+    def list(self, zone: Optional[str] = None) -> None:
         """Show a list of all physical hosts.
 
         Can be filtered by zone.
@@ -129,7 +134,7 @@ class HostCommands(object):
         services = objects.ServiceList.get_all(ctxt)
         if zone:
             services = [s for s in services if s.availability_zone == zone]
-        hosts = []
+        hosts: list[dict[str, Any]] = []
         for srv in services:
             if not [h for h in hosts if h['host'] == srv['host']]:
                 hosts.append(srv)
@@ -152,14 +157,10 @@ class DbCommands(object):
     # preceed any element of the "online_migrations" tuple, like this:
     #    # Added in Queens remove in Rocky
     #    db.service_uuids_online_data_migration,
-    online_migrations = (
-        # TODO: (Z Release) Remove next line and this comment
-        # TODO: (Y Release) Uncomment next line and remove this comment
-        # db.remove_temporary_admin_metadata_data_migration,
-
-        # TODO: (Y Release) Remove next 2 line and this comment
-        db.volume_use_quota_online_data_migration,
-        db.snapshot_use_quota_online_data_migration,
+    online_migrations: Tuple[Callable[[context.RequestContext, int],
+                                      Tuple[int, int]], ...] = (
+        # TODO: (D Release) Remove next line and this comment
+        db.remove_temporary_admin_metadata_data_migration,
     )
 
     def __init__(self):
@@ -172,7 +173,9 @@ class DbCommands(object):
           help='Update RPC and Objects versions when doing offline upgrades, '
                'with this we no longer need to restart the services twice '
                'after the upgrade to prevent ServiceTooOld exceptions.')
-    def sync(self, version=None, bump_versions=False):
+    def sync(self,
+             version: Optional[int] = None,
+             bump_versions: bool = False) -> None:
         """Sync the database up to the most recent version."""
         if version is not None and version > db.MAX_INT:
             print(_('Version should be less than or equal to '
@@ -199,13 +202,13 @@ class DbCommands(object):
             print(_('Error during service version bump: %s') % ex)
             sys.exit(2)
 
-    def version(self):
+    def version(self) -> None:
         """Print the current database version."""
         print(db_migration.db_version())
 
     @args('age_in_days', type=int,
           help='Purge deleted rows older than age in days')
-    def purge(self, age_in_days):
+    def purge(self, age_in_days: int) -> None:
         """Purge deleted rows older than a given age from cinder tables."""
         age_in_days = int(age_in_days)
         if age_in_days < 0:
@@ -223,7 +226,9 @@ class DbCommands(object):
                     "logs for more details."))
             sys.exit(1)
 
-    def _run_migration(self, ctxt, max_count):
+    def _run_migration(self,
+                       ctxt: context.RequestContext,
+                       max_count: int) -> Tuple[dict, bool]:
         ran = 0
         exceptions = False
         migrations = {}
@@ -254,7 +259,7 @@ class DbCommands(object):
 
     @args('--max_count', metavar='<number>', dest='max_count', type=int,
           help='Maximum number of objects to consider.')
-    def online_data_migrations(self, max_count=None):
+    def online_data_migrations(self, max_count: Optional[int] = None) -> None:
         """Perform online data migrations for the release in batches."""
         ctxt = context.get_admin_context()
         if max_count is not None:
@@ -269,7 +274,7 @@ class DbCommands(object):
 
         ran = None
         exceptions = False
-        migration_info = {}
+        migration_info: dict[str, Any] = {}
         while ran is None or ran != 0:
             migrations, exceptions = self._run_migration(ctxt, max_count)
             ran = 0
@@ -311,8 +316,10 @@ class DbCommands(object):
           help='Change the active backend ID (default: %(default)s).')
     @args('--backend-host', required=True,
           help='The backend host name.')
-    def reset_active_backend(self, enable_replication, active_backend_id,
-                             backend_host):
+    def reset_active_backend(self,
+                             enable_replication: bool,
+                             active_backend_id: Optional[str],
+                             backend_host: str) -> None:
         """Reset the active backend for a host."""
 
         ctxt = context.get_admin_context()
@@ -334,9 +341,9 @@ class QuotaCommands(object):
         pass
 
     @args('--project-id', default=None,
-          help=('The ID of the project where we want to sync the quotas '
+          help=('The ID of the project where we want to check the quotas '
                 '(defaults to all projects).'))
-    def check(self, project_id):
+    def check(self, project_id: Optional[str]) -> None:
         """Check if quotas and reservations are correct
 
         This action checks quotas and reservations, for a specific project or
@@ -356,7 +363,7 @@ class QuotaCommands(object):
     @args('--project-id', default=None,
           help=('The ID of the project where we want to sync the quotas '
                 '(defaults to all projects).'))
-    def sync(self, project_id):
+    def sync(self, project_id: Optional[str]) -> None:
         """Fix quotas and reservations
 
         This action refreshes existing quota usage and reservation count for a
@@ -373,12 +380,14 @@ class QuotaCommands(object):
         self._check_sync(project_id, do_fix=True)
 
     @db_api.main_context_manager.reader
-    def _get_quota_projects(self, context, project_id):
+    def _get_quota_projects(self,
+                            ctxt: context.RequestContext,
+                            project_id: Optional[str]) -> list[str]:
         """Get project ids that have quota_usage entries."""
         if project_id:
             model = models.QuotaUsage
             # If the project does not exist
-            if not context.session.query(
+            if not ctxt.session.query(
                 db_api.sql.exists()
                 .where(
                     db_api.and_(
@@ -394,31 +403,35 @@ class QuotaCommands(object):
                 return []
             return [project_id]
 
-        projects = db_api.model_query(
-            context,
-            models.QuotaUsage,
-            read_deleted="no"
-        ).with_entities('project_id').distinct().all()
+        projects = db_api.get_projects(ctxt,
+                                       models.QuotaUsage,
+                                       read_deleted="no")
         project_ids = [row.project_id for row in projects]
         return project_ids
 
-    def _get_usages(self, context, resources, project_id):
+    def _get_usages(self,
+                    ctxt: context.RequestContext,
+                    resources,
+                    project_id: str) -> list:
         """Get data necessary to check out of sync quota usage.
 
-        Returns a list QuotaUsage instances for the specific project
+        Returns a list of QuotaUsage instances for the specific project
         """
         usages = db_api.model_query(
-            context,
+            ctxt,
             db_api.models.QuotaUsage,
             read_deleted="no",
         ).filter_by(project_id=project_id).with_for_update().all()
         return usages
 
-    def _get_reservations(self, context, project_id, usage_id):
+    def _get_reservations(self,
+                          ctxt: context.RequestContext,
+                          project_id: str,
+                          usage_id: str) -> list:
         """Get reservations for a given project and usage id."""
         reservations = (
             db_api.model_query(
-                context,
+                ctxt,
                 models.Reservation,
                 read_deleted="no",
             )
@@ -428,7 +441,10 @@ class QuotaCommands(object):
         )
         return reservations
 
-    def _check_duplicates(self, context, usages, do_fix):
+    def _check_duplicates(self,
+                          ctxt: context.RequestContext,
+                          usages,
+                          do_fix: bool) -> tuple[list, bool]:
         """Look for duplicated quota used entries (bug#1484343)
 
         If we have duplicates and we are fixing them, then we reassign the
@@ -453,7 +469,7 @@ class QuotaCommands(object):
                     reassigned = 0
                     for usage in resource_usages[1:]:
                         reservations = self._get_reservations(
-                            context,
+                            ctxt,
                             usage.project_id,
                             usage.id,
                         )
@@ -462,7 +478,7 @@ class QuotaCommands(object):
                             reservation.usage_id = keep_usage.id
                         keep_usage.in_use += usage.in_use
                         keep_usage.reserved += usage.reserved
-                        usage.delete(context.session)
+                        usage.delete(ctxt.session)
                     print('duplicates removed & %s reservations reassigned' %
                           reassigned)
                 else:
@@ -470,7 +486,7 @@ class QuotaCommands(object):
             result.append(keep_usage)
         return result, duplicates_found
 
-    def _check_sync(self, project_id, do_fix):
+    def _check_sync(self, project_id: Optional[str], do_fix: bool) -> bool:
         """Check the quotas and reservations optionally fixing them."""
 
         ctxt = context.get_admin_context()
@@ -496,7 +512,11 @@ class QuotaCommands(object):
         return discrepancy
 
     @db_api.main_context_manager.reader
-    def _check_project_sync(self, context, project, do_fix, resources):
+    def _check_project_sync(self,
+                            ctxt: context.RequestContext,
+                            project: str,
+                            do_fix: bool,
+                            resources) -> bool:
         print('Processing quota usage for project %s' % project)
 
         discrepancy = False
@@ -507,11 +527,11 @@ class QuotaCommands(object):
         # running Cinder services.
 
         # We only want to sync existing quota usage rows
-        usages = self._get_usages(context, resources, project)
+        usages = self._get_usages(ctxt, resources, project)
 
         # Check for duplicated entries (bug#1484343)
         usages, duplicates_found = self._check_duplicates(
-            context, usages, do_fix,
+            ctxt, usages, do_fix,
         )
         if duplicates_found:
             discrepancy = True
@@ -521,7 +541,7 @@ class QuotaCommands(object):
             resource_name = usage.resource
             # Get the correct value for this quota usage resource
             updates = db_api._get_sync_updates(
-                context,
+                ctxt,
                 project,
                 resources,
                 resource_name,
@@ -537,7 +557,7 @@ class QuotaCommands(object):
                     usage.in_use = in_use
 
             reservations = self._get_reservations(
-                context,
+                ctxt,
                 project,
                 usage.id,
             )
@@ -579,7 +599,7 @@ class VolumeCommands(object):
 
     @args('volume_id',
           help='Volume ID to be deleted')
-    def delete(self, volume_id):
+    def delete(self, volume_id: str) -> None:
         """Delete a volume, bypassing the check that it must be available."""
         ctxt = context.get_admin_context()
         volume = objects.Volume.get_by_id(ctxt, volume_id)
@@ -604,7 +624,7 @@ class VolumeCommands(object):
                                                'the format host@backend#pool')
     @args('--newhost', required=True, help='New volume host name in the '
                                            'format host@backend#pool')
-    def update_host(self, currenthost, newhost):
+    def update_host(self, currenthost: str, newhost: str) -> None:
         """Modify the host name associated with a volume.
 
         Particularly to recover from cases where one has moved
@@ -618,6 +638,16 @@ class VolumeCommands(object):
             db.volume_update(ctxt, v['id'],
                              {'host': newhost})
 
+    def update_service(self):
+        """Modify the service uuid associated with a volume.
+
+        In certain upgrade cases, we create new cinder services and delete the
+        records of old ones, however, the volumes created with old service
+        still contain the service uuid of the old services.
+        """
+        ctxt = context.get_admin_context()
+        db.volume_update_all_by_service(ctxt)
+
 
 class ConfigCommands(object):
     """Class for exposing the flags defined by flag_file(s)."""
@@ -627,7 +657,7 @@ class ConfigCommands(object):
 
     @args('param', nargs='?', default=None,
           help='Configuration parameter to display (default: %(default)s)')
-    def list(self, param=None):
+    def list(self, param: Optional[str] = None) -> None:
         """List parameters configured for cinder.
 
         Lists all parameters configured for cinder unless an optional argument
@@ -646,7 +676,7 @@ class ConfigCommands(object):
 class BackupCommands(object):
     """Methods for managing backups."""
 
-    def list(self):
+    def list(self) -> None:
         """List all backups.
 
         List all backups (including ones in progress) and the host
@@ -683,7 +713,7 @@ class BackupCommands(object):
 
     @args('--currenthost', required=True, help='Existing backup host name')
     @args('--newhost', required=True, help='New backup host name')
-    def update_backup_host(self, currenthost, newhost):
+    def update_backup_host(self, currenthost: str, newhost: str) -> None:
         """Modify the host name associated with a backup.
 
         Particularly to recover from cases where one has moved
@@ -738,7 +768,7 @@ class ServiceCommands(BaseCommand):
           help='Service to delete from the host.')
     @args('host_name', type=str,
           help='Host from which to remove the service.')
-    def remove(self, binary, host_name):
+    def remove(self, binary: str, host_name: str) -> Optional[int]:
         """Completely removes a service."""
         ctxt = context.get_admin_context()
         try:
@@ -753,10 +783,12 @@ class ServiceCommands(BaseCommand):
         print(_("Service %(service)s on host %(host)s removed.") %
               {'service': binary, 'host': host_name})
 
+        return None
+
 
 class ClusterCommands(BaseCommand):
     """Methods for managing clusters."""
-    def list(self):
+    def list(self) -> None:
         """Show a list of all cinder services."""
         ctxt = context.get_admin_context()
         clusters = objects.ClusterList.get_all(ctxt, services_summary=True)
@@ -783,7 +815,10 @@ class ClusterCommands(BaseCommand):
     @args('binary', type=str,
           help='Service to delete from the cluster.')
     @args('cluster-name', type=str, help='Cluster to delete.')
-    def remove(self, recursive, binary, cluster_name):
+    def remove(self,
+               recursive: bool,
+               binary: str,
+               cluster_name: str) -> Optional[int]:
         """Completely removes a cluster."""
         ctxt = context.get_admin_context()
         try:
@@ -812,13 +847,18 @@ class ClusterCommands(BaseCommand):
                      'removed.') % {'msg': msg, 'num': len(cluster.services)})
         print(msg)
 
+        return None
+
     @args('--full-rename', dest='partial',
           action='store_false', default=True,
           help='Do full cluster rename instead of just replacing provided '
                'current cluster name and preserving backend and/or pool info.')
     @args('current', help='Current cluster name.')
     @args('new', help='New cluster name.')
-    def rename(self, partial, current, new):
+    def rename(self,
+               partial: bool,
+               current: Optional[str],
+               new: Optional[str]) -> Optional[int]:
         """Rename cluster name for Volumes and Consistency Groups.
 
         Useful when you want to rename a cluster, particularly when the
@@ -851,13 +891,15 @@ class ClusterCommands(BaseCommand):
             print(msg % {'current': current})
             return 2
 
+        return None
+
 
 class ConsistencyGroupCommands(object):
     """Methods for managing consistency groups."""
 
     @args('--currenthost', required=True, help='Existing CG host name')
     @args('--newhost', required=True, help='New CG host name')
-    def update_cg_host(self, currenthost, newhost):
+    def update_cg_host(self, currenthost: str, newhost: str) -> None:
         """Modify the host name associated with a Consistency Group.
 
         Particularly to recover from cases where one has moved
@@ -878,7 +920,9 @@ class UtilCommands(object):
     """Generic utils."""
 
     @staticmethod
-    def _get_resources_locks():
+    def _get_resources_locks() -> Tuple[collections.defaultdict,
+                                        collections.defaultdict,
+                                        collections.defaultdict]:
         """Get all vol/snap/backup file lock paths."""
         backup_locks_prefix = 'cinder-cleanup_incomplete_backups_'
         oslo_dir = os.path.abspath(cfg.CONF.oslo_concurrency.lock_path)
@@ -890,8 +934,8 @@ class UtilCommands(object):
             if tooz_dir != oslo_dir:
                 filenames += glob.glob(os.path.join(tooz_dir, 'cinder-*'))
 
-        volumes = collections.defaultdict(list)
-        snapshots = collections.defaultdict(list)
+        volumes: collections.defaultdict = collections.defaultdict(list)
+        snapshots: collections.defaultdict = collections.defaultdict(list)
         backups = collections.defaultdict(list)
         matcher = re.compile('.*?([0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-'
                              '[0-9a-f]{4}-[0-9a-f]{12}).*?', re.IGNORECASE)
@@ -908,7 +952,7 @@ class UtilCommands(object):
 
         return volumes, snapshots, backups
 
-    def _exclude_running_backups(self, backups):
+    def _exclude_running_backups(self, backups: dict) -> None:
         """Remove backup entries from the dict for running backup services."""
         for backup_pgrp in list(backups.keys()):
             # The PGRP is the same as the PID of the parent process, so we know
@@ -928,7 +972,7 @@ class UtilCommands(object):
     @args('--services-offline', dest='online',
           action='store_false', default=True,
           help='All locks can be deleted as Cinder services are not running.')
-    def clean_locks(self, online):
+    def clean_locks(self, online: bool) -> None:
         """Clean file locks for vols, snaps, and backups on the current host.
 
         Should be run on any host where we are running a Cinder service (API,
@@ -955,6 +999,8 @@ class UtilCommands(object):
         self.ctxt = context.get_admin_context()
         # Find volume and snapshots ids, and backups PGRP based on the existing
         # file locks
+        volumes: Union[collections.defaultdict, dict]
+        snapshots: Union[collections.defaultdict, dict]
         volumes, snapshots, backups = self._get_resources_locks()
 
         # If services are online we cannot delete locks for existing resources
@@ -999,7 +1045,7 @@ CATEGORIES = {
 }
 
 
-def methods_of(obj):
+def methods_of(obj) -> list:
     """Return non-private methods from an object.
 
     Get all callable methods of an object that don't start with underscore
@@ -1007,13 +1053,14 @@ def methods_of(obj):
     """
     result = []
     for i in dir(obj):
-        if isinstance(getattr(obj, i),
-                      collections_abc.Callable) and not i.startswith('_'):
+        if (isinstance(getattr(obj, i),
+                       collections_abc.Callable) and  # type: ignore
+                not i.startswith('_')):
             result.append((i, getattr(obj, i)))
     return result
 
 
-def missing_action(help_func):
+def missing_action(help_func: Callable) -> Callable:
     def wrapped():
         help_func()
         exit(2)
@@ -1033,7 +1080,7 @@ def add_command_parsers(subparsers):
         for (action, action_fn) in methods_of(command_object):
             parser = category_subparsers.add_parser(action)
 
-            action_kwargs = []
+            action_kwargs: list = []
             for args, kwargs in getattr(action_fn, 'args', []):
                 parser.add_argument(*args, **kwargs)
 
