@@ -21,8 +21,8 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import importutils
 from oslo_utils import units
-import six
 
+from cinder.common import constants
 from cinder import exception
 from cinder.i18n import _
 from cinder import interface
@@ -93,7 +93,7 @@ class StorPoolDriver(driver.VolumeDriver):
     """
 
     VERSION = '1.2.3'
-    CI_WIKI_NAME = 'StorPool_CI'
+    CI_WIKI_NAME = 'StorPool_distributed_storage_CI'
 
     def __init__(self, *args, **kwargs):
         super(StorPoolDriver, self).__init__(*args, **kwargs)
@@ -108,7 +108,7 @@ class StorPoolDriver(driver.VolumeDriver):
         return storpool_opts
 
     def _backendException(self, e):
-        return exception.VolumeBackendAPIException(data=six.text_type(e))
+        return exception.VolumeBackendAPIException(data=str(e))
 
     def _template_from_volume(self, volume):
         default = self.configuration.storpool_template
@@ -301,14 +301,15 @@ class StorPoolDriver(driver.VolumeDriver):
                        ) for t in templates]
 
         self._stats = {
+            # Basic driver properties
             'volume_backend_name': self.configuration.safe_get(
                 'volume_backend_name') or 'storpool',
             'vendor_name': 'StorPool',
             'driver_version': self.VERSION,
-            'storage_protocol': 'storpool',
-
+            'storage_protocol': constants.STORPOOL,
+            # Driver capabilities
             'sparse_copy_volume': True,
-
+            # The actual pools data
             'pools': pools
         }
 
@@ -408,7 +409,8 @@ class StorPoolDriver(driver.VolumeDriver):
                     '%(vol)s: %(err)s',
                     {'name': name, 'vol': volname, 'err': e})
 
-    def copy_image_to_volume(self, context, volume, image_service, image_id):
+    def copy_image_to_volume(self, context, volume, image_service, image_id,
+                             disable_sparse=False):
         req_id = context.request_id
         name = self._attach.volumeName(volume['id'])
         self._attach.add(req_id, {
@@ -419,7 +421,8 @@ class StorPoolDriver(driver.VolumeDriver):
         })
         try:
             return super(StorPoolDriver, self).copy_image_to_volume(
-                context, volume, image_service, image_id)
+                context, volume, image_service, image_id,
+                disable_sparse=disable_sparse)
         finally:
             self._attach.remove(req_id)
 
