@@ -15,7 +15,7 @@ from __future__ import annotations
 import binascii
 import traceback
 import typing
-from typing import Any, Optional  # noqa: H301
+from typing import Any, Optional
 
 from castellan import key_manager
 import os_brick.initiator.connectors
@@ -146,7 +146,7 @@ class OnFailureRescheduleTask(flow_utils.CinderTask):
                           volume.id)
 
     def _reschedule(self, context, cause, request_spec, filter_properties,
-                    volume):
+                    volume) -> None:
         """Actions that happen during the rescheduling attempt occur here."""
 
         create_volume = self.scheduler_rpcapi.create_volume
@@ -170,8 +170,8 @@ class OnFailureRescheduleTask(flow_utils.CinderTask):
             # Stringify to avoid circular ref problem in json serialization
             retry_info['exc'] = traceback.format_exception(*cause.exc_info)
 
-        return create_volume(context, volume, request_spec=request_spec,
-                             filter_properties=filter_properties)
+        create_volume(context, volume, request_spec=request_spec,
+                      filter_properties=filter_properties)
 
     def _post_reschedule(self, volume):
         """Actions that happen after the rescheduling attempt occur here."""
@@ -598,17 +598,18 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                     # and is detected as another format.
                     raise exception.Invalid()
 
-                if encryption['provider'] == 'luks':
+                encryption_provider = encryption['provider']
+                if encryption_provider == 'luks':
                     # Force ambiguous "luks" provider to luks1 for
                     # compatibility with new versions of cryptsetup.
-                    encryption['provider'] = 'luks1'
+                    encryption_provider = 'luks1'
 
                 (out, err) = utils.execute(
                     'cryptsetup',
                     '--batch-mode',
                     'luksFormat',
                     '--force-password',
-                    '--type', encryption['provider'],
+                    '--type', encryption_provider,
                     '--cipher', encryption['cipher'],
                     '--key-size', str(encryption['key_size']),
                     '--key-file=-',
@@ -1149,6 +1150,10 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
         try:
             ret = self.driver.create_volume_from_backup(volume, backup)
             need_update_volume = True
+            LOG.info("Created volume %(volume_id)s from backup %(backup_id)s "
+                     "successfully.",
+                     {'volume_id': volume.id,
+                      'backup_id': backup_id})
 
         except NotImplementedError:
             LOG.info("Backend does not support creating volume from "
@@ -1170,15 +1175,15 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
             backup.update(updates)
             backup.save()
 
+            LOG.info("Raw volume %(volume_id)s created.  Calling "
+                     "restore_backup %(backup_id)s to complete restoration.",
+                     {'volume_id': volume.id,
+                      'backup_id': backup_id})
             backuprpcapi = backup_rpcapi.BackupAPI()
             backuprpcapi.restore_backup(context, backup.host, backup,
-                                        volume.id)
+                                        volume.id, volume_is_new=True)
             need_update_volume = False
 
-        LOG.info("Created volume %(volume_id)s from backup %(backup_id)s "
-                 "successfully.",
-                 {'volume_id': volume.id,
-                  'backup_id': backup_id})
         return ret, need_update_volume
 
     def _create_raw_volume(self,
@@ -1290,6 +1295,7 @@ class CreateVolumeOnFinishTask(NotifyVolumeActionTask):
             'migration_target_creating': 'migration_target',
         }
 
+    @typing.no_type_check
     def execute(self, context, volume, volume_spec):
         need_update_volume = volume_spec.pop('need_update_volume', True)
         if not need_update_volume:
@@ -1371,8 +1377,7 @@ def get_flow(context, manager, db, driver, scheduler_rpcapi, host, volume,
     volume_flow.add(ExtractVolumeSpecTask(db))
     # Temporary volumes created during migration should not be notified
     end_notify_suffix = None
-    # TODO: (Y release) replace check with: if volume.use_quota:
-    if volume.use_quota or not volume.is_migration_target():
+    if volume.use_quota:
         volume_flow.add(NotifyVolumeActionTask(db, 'create.start'))
         end_notify_suffix = 'create.end'
     volume_flow.add(CreateVolumeFromSpecTask(manager,
